#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:u:"


/********************************************************
**
** Test Module: SetPropDelay.c
**
** Purpose:     Set the signal delay (in micro-seconds) 
**              that the device should account for due to
**              extended cable lengths from the time-input 
**              source.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int32_t         uSecDelay = 0;

    /*
     * parse the args
     */
    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'u' :
                uSecDelay = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT,
		devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * Set the propagation delay time (in microseconds) on the board.
     */
    int done = 0;
    while ( !done ) 
    {
        if ( (rc = TPRO_setPropDelayCorr( hnd, &uSecDelay )) != TPRO_SUCCESS )
        {
            char emsg[MAX_ERR_MESG_LENGTH];
            done = 1;

            switch (rc)
            {
                case TPRO_HANDLE_ERR:
                    errMsg = "TPRO_HANDLE_ERR";
                    break;
                case TPRO_COMM_ERR:
                    errMsg = "TPRO_COMM_ERR";
                    break;
                case TPRO_DELAY_PARM_ERR:
                    (void)sprintf_s( emsg, MAX_ERR_MESG_LENGTH, "TPRO_DELAY_PARM_ERR uSec <%d>\n", uSecDelay );
                    errMsg = emsg;
                    break;
                case TPRO_DEV_BUSY:
                    errMsg = "TPRO_DEV_BUSY waiting...";
                    done = 0;
                    break;
                default:
                    errMsg = "Unknown error";
                    break;
            }

            (void) printf("%s: <%s> %s rc: <%d>.\n", 
                        pgmname, fullDevName, errMsg, rc);

            ret = 1;
    
        }
        else
        {
            /*
             * success:  print out the delay that was set
             */
            (void)printf("<%s> %s: Done uSec <%d>\n", fullDevName, 
                                            pgmname, uSecDelay );
            done = 1;
            ret = 0;
        }

    } /* end while busy... */

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] [-u MICRO_SEC ] \n\n", pgmName );
    (void)printf("\t-u   microseconds (e.g. -1000 to 8999 .0)\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n\n");
    (void)printf("\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
    (void)printf("\t-h   help message\n");
}
